@@ -177,9 +177,11 @@ void SingleCellTileDBIterator::begin_new_query_column_interval(TileDB_CTX* tiled
     std::vector<const char*>* attribute_names) {
   do {
     //This can happen iff in the previous iteration intersecting intervals were searched OR
-    //full array scan is requested and this is the first time a read is being performed
+    //full array scan is requested and this is the first time a read is being performed OR
+    //if only a single pass simple traversal is desired
     if (m_in_find_intersecting_intervals_mode
-        || (m_first_read_from_TileDB && (m_query_config->get_num_column_intervals() == 0u)))
+        || (m_first_read_from_TileDB && (m_query_config->get_num_column_intervals() == 0u
+                                         || m_query_config->bypass_intersecting_intervals_phase())))
       move_into_simple_traversal_mode();
     else {
       reset_for_next_query_interval();
@@ -193,7 +195,8 @@ void SingleCellTileDBIterator::begin_new_query_column_interval(TileDB_CTX* tiled
           return;
         }
       }
-      m_in_find_intersecting_intervals_mode = true;
+      m_in_simple_traversal_mode = m_query_config->bypass_intersecting_intervals_phase();
+      m_in_find_intersecting_intervals_mode = !m_query_config->bypass_intersecting_intervals_phase();
     }
     auto smallest_row_idx_in_array = m_query_config->get_smallest_row_idx_in_array();
     //Range of the query
@@ -204,8 +207,9 @@ void SingleCellTileDBIterator::begin_new_query_column_interval(TileDB_CTX* tiled
     if (m_query_config->get_num_column_intervals() > 0u) {
       assert(m_query_column_interval_idx < m_query_config->get_num_column_intervals());
       query_range[2] = m_query_config->get_column_begin(m_query_column_interval_idx);
-      if (m_in_simple_traversal_mode)
+      if (m_in_simple_traversal_mode) {
         query_range[3] = m_query_config->get_column_end(m_query_column_interval_idx);
+      }
     }
     auto status = -1;
     if (m_tiledb_array == 0) {
@@ -505,6 +509,31 @@ bool SingleCellTileDBIterator::advance_to_next_useful_cell(const uint64_t min_nu
 #endif
     advance_fields_other_than_coords_END(num_cells_incremented);
   }
+
+  // Evaluate filter expression if necessary
+  m_cell_evaluated_with_filter_expression = true;
+  if (m_query_config->get_query_filter().size() > 0) {
+    const auto& attribute_ids = m_query_config->get_query_attributes_schema_idxs();
+    std::vector<void *> buffers;
+    std::vector<size_t> buffer_sizes;
+    std::vector<int64_t> positions;
+    for (auto i=0; i<m_fields.size(); i++) {
+      auto& genomicsdb_columnar_field = m_fields[i];
+      auto buffer = get_buffer_and_index(i);
+      if (genomicsdb_columnar_field.is_variable_length_field()) {
+        buffers.push_back(const_cast<GenomicsDBBuffer *>(buffer.first)->get_offsets_pointer());
+        buffer_sizes.push_back(buffer.first->get_offsets_size_in_bytes());
+        positions.push_back((int64_t)buffer.second);
+      }
+      buffers.push_back(const_cast<uint8_t *>(buffer.first->get_raw_pointer()));
+      buffer_sizes.push_back(buffer.first->get_data_vector_size_in_bytes());
+      positions.push_back((int64_t)buffer.second);
+    }
+    if (tiledb_array_evaluate_cell(m_tiledb_array, buffers.data(), buffer_sizes.data(), positions.data()) != TILEDB_OK) {
+      m_cell_evaluated_with_filter_expression = false;
+    }
+  }
+
   return true;
 }
 
@@ -642,7 +671,7 @@ void SingleCellTileDBIterator::advance_fields_other_than_coords_END(const uint64
   assert(m_END_query_idx == 0u);
   for (auto i=0u; i<m_query_attribute_idx_vec.size(); ++i)
     m_query_attribute_idx_vec[i] = i+1u;  //END is the first field - ignore
-  //-2 - ignore coords and END
+  //-2 - ignore coords as well
   m_query_attribute_idx_num_cells_to_increment_vec.resize(m_fields.size()-2u);
   //For all fields, #cells to skip == num_cells_to_increment initially
   m_query_attribute_idx_num_cells_to_increment_vec.assign(
@@ -656,7 +685,7 @@ void SingleCellTileDBIterator::advance_fields_other_than_coords_END(const uint64
     assert(!m_done_reading_from_TileDB
            && m_query_column_interval_idx == curr_query_column_interval_idx);
 #ifdef DEBUG
-    //After the skip cells API is implemented, there shouldn't be any attributes
+    //After the TileDB read with skip cells, there shouldn't be any attributes
     //whose cells need to be skipped after a call to read_from_TileDB()
     for (auto i=0u; i<m_query_attribute_idx_vec.size(); ++i)
       assert(m_query_attribute_idx_num_cells_to_increment_vec[i] == 0u);
