@@ -1,7 +1,7 @@
 /**
  * The MIT License (MIT)
  * Copyright (c) 2016-2017 Intel Corporation
- * Copyright (c) 2018-2022 Omics Data Automation, Inc.
+ * Copyright (c) 2018-2023 Omics Data Automation, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy of
  * this software and associated documentation files (the "Software"), to deal in
@@ -534,7 +534,7 @@ VariantStorageManager::VariantStorageManager(const std::string& workspace, const
     }
   } else {
     logger.fatal(VariantStorageManagerException(
-                     logger.format("Workspace {} does not seem to exist{}", workspace, PRINT_TILEDB_ERRMSG)));
+                     logger.format("Could not open workspace {}{}", workspace, PRINT_TILEDB_ERRMSG)));
   }
 }
 
@@ -544,8 +544,11 @@ int VariantStorageManager::open_array(const std::string& array_name, const VidMa
   auto mode_int = (*mode_iter).second;
 
   std::string array_path = SLASHIFY(m_workspace) + array_name;
-  if (mode_int == TILEDB_ARRAY_READ && !is_array(m_tiledb_ctx, array_path)) {
-    logger.error("Array {} in workspace {} does not seem to exist{}", array_name,  m_workspace, PRINT_TILEDB_ERRMSG);
+  if (!is_array(m_tiledb_ctx, array_path)) {
+    if (mode_int == TILEDB_ARRAY_READ) {
+      logger.error("Could not open array {} in workspace {}{}",
+                   array_name,  m_workspace, PRINT_TILEDB_ERRMSG);
+    }
     return -1;
   }
 
