@@ -1,7 +1,8 @@
 #!/bin/bash
 #
 # The MIT License (MIT)
-# Copyright (c) 2021-2022 Omics Data Automation Inc.
+# Copyright (c) 2021-2023 Omics Data Automation Inc.
+# Copyright (c) 2023 da̅tma, inc.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy of
 # this software and associated documentation files (the "Software"), to deal in
@@ -25,10 +26,11 @@
 #    vcf2genomicsdb
 #    gt_mpi_gather
 # $1 contains the test vcfs - t0.vcf.gz, t1.vcf.gz and t2.vcf.gz
-if [[ $# -ne 2 ]]; then
-  echo "Usage: ./test_tools.sh <vcfs_dir> <install_dir>"
+if [[ $# -lt 2 ]]; then
+  echo "Usage: ./test_tools.sh <vcfs_dir> <install_dir> <disable_mpi>"
   echo "<vcf_dir> can be a local or remote directory containing the test vcfs -"
   echo "            t0.vcf.gz, t1.vcf.gz and t2.vcf.gz"
+  echo "optional <disable_mpi>, if set gt_mpi_gather_tests are not run"
   exit 1
 fi
 
@@ -36,13 +38,16 @@ if [[ -n $2 ]]; then
   PATH=$2/bin:$2:$PATH
 fi
 
+if [[ $# -ge 3 ]]; then
+  DISABLE_MPI=$3
+else
+  DISABLE_MPI=0
+fi
+
 VCFS_DIR=$(cd $1 && pwd)
 TEMP_DIR=$(mktemp -d -t test_tools-XXXXXXXXXX)
 
 WORKSPACE=$TEMP_DIR/ws
-LOADER_JSON=$TEMP_DIR/ws/loader.json
-CALLSET_MAPPING_JSON=$TEMP_DIR/ws/callset_mapping.json
-TEMPLATE_HEADER=$TESTS_DIR/ws/vcf_header.vcf
 
 REFERENCE_GENOME=$VCFS_DIR/../chr1_10MB.fasta.gz
 
@@ -97,6 +102,7 @@ create_template_loader_json() {
 {
     "treat_deletions_as_intervals": true,
     "compress_tiledb_array": true,
+    "produce_combined_vcf": false,
     "produce_tiledb_array": true,
     "size_per_column_partition": 700,
     "delete_and_create_tiledb_array": true,
@@ -123,7 +129,7 @@ EOF
 }
 
 # create_query_json
-#    (Optional) $1 reference file, only necessary when producing vcfs
+#    (Optional) $1 additional json entries
 create_query_json() {
   QUERY_JSON=$TEMP_DIR/query_json_$RANDOM
   cat > $QUERY_JSON  << EOF
@@ -143,7 +149,7 @@ create_query_json() {
 EOF
   if [[ $# -ge 1 ]]; then
     cat >> $QUERY_JSON  << EOF
-    "reference_genome" : "$1",
+    $1,
 EOF
   fi
   cat >> $QUERY_JSON << EOF
@@ -152,6 +158,34 @@ EOF
 EOF
 }
 
+# create_query_json with intervals
+#    $1 is the query interval specification
+#    (Optional) $2 additional json entries
+create_query_json_with_intervals() {
+  QUERY_JSON=$TEMP_DIR/query_json_$RANDOM
+  cat > $QUERY_JSON  << EOF
+{
+    "workspace": "$WORKSPACE",
+    "generate_array_name_from_partition_bounds": true,
+    $1,
+    "query_row_ranges": [{
+        "range_list": [{
+            "low" : 0,
+            "high": 3
+        }]
+     }],
+EOF
+  if [[ $# -ge 2 ]]; then
+    cat >> $QUERY_JSON  << EOF
+    $2,
+EOF
+  fi
+  cat >> $QUERY_JSON << EOF
+    "attributes" : [ "REF", "ALT", "BaseQRankSum", "MQ", "RAW_MQ", "MQ0", "ClippingRankSum", "MQRankSum", "ReadPosRankSum", "DP", "GT", "GQ", "SB", "AD", "PL", "DP_FORMAT", "MIN_DP", "PGT" ]
+}
+EOF
+}
+
 # run_command
 #    $1 : command to be executed
 #    $2 : optional - 0(default) if command should return successfully
@@ -179,13 +213,28 @@ run_command() {
   fi
 }
 
+time_command() {
+  if [[ $(uname) == "Darwin" ]]; then
+    DT="date -u +%s"
+  else
+    DT="date -u +%s%N"
+  fi
+  START=$($DT)
+  run_command "$@"
+  END=$($DT)
+  TIME=$(($END-$START))
+}
+
+OK=0
 ERR=1
 
 # Sanity Checks for Tools
 sanity_check "vcf2genomicsdb_init"
 sanity_check "vcf2genomicsdb"
 sanity_check "consolidate_genomicsdb_array"
-sanity_check "gt_mpi_gather"
+if [[ $DISABLE_MPI == 0 ]]; then
+  sanity_check "gt_mpi_gather"
+fi
 
 WORKSPACE=$TEMP_DIR/ws_$RANDOM
 run_command "vcf2genomicsdb_init -w $WORKSPACE" ERR
@@ -279,7 +328,7 @@ run_command_and_check_results "vcf2genomicsdb_init -w $WORKSPACE -o -S $SAMPLE_D
 # Template loader json
 create_template_loader_json
 run_command_and_check_results "vcf2genomicsdb_init -w $WORKSPACE -S $SAMPLE_DIR -o -t $TEMPLATE" 2 85 24 85 "#18"
-assert_true $(grep '"segment_size": 400' $WORKSPACE/loader.json | wc -l) 1 "Test #16 segment_size from template loader json was not applied"
+assert_true $(grep '"segment_size": 400' $WORKSPACE/loader.json | wc -l) 1 "Test #18 segment_size from template loader json was not applied"
 
 # Fail if same field in INFO and FORMAT have different types
 create_sample_list inconsistent_DP_t0.vcf.gz
@@ -302,7 +351,32 @@ run_command_and_check_results "vcf2genomicsdb_init -w $WORKSPACE -S $SAMPLE_DIR
 create_template_loader_json $TILEDB_COMPRESSION_ZLIB
 run_command_and_check_results "vcf2genomicsdb_init -w $WORKSPACE -S $SAMPLE_DIR -o -t $TEMPLATE" 2 85 24 85 "$23"
 
+# Fail if generate_array_name_from_partition_bounds(not supported yet!) is set
+create_template_loader_json
+run_command "vcf2genomicsdb_init -w $WORKSPACE -s $SAMPLE_LIST -o -t $TEMPLATE"
+sed -i -e 's/"array_name": .*,/"generate_array_name_from_partition_bounds": true,/g' $WORKSPACE/loader.json
+run_command "vcf2genomicsdb $WORKSPACE/loader.json" ERR
+
+# Try running with produce_combined_vcf
+create_template_loader_json
+run_command "vcf2genomicsdb_init -w $WORKSPACE -s $SAMPLE_LIST -o -t $TEMPLATE"
+sed -i -e 's/"produce_combined_vcf": false/"produce_combined_vcf": true/g' $WORKSPACE/loader.json
+run_command "vcf2genomicsdb $WORKSPACE/loader.json" 134
+sed -i -e 's|"produce_tiledb_array": true|"reference_genome": "'${REFERENCE_GENOME}'"|g' $WORKSPACE/loader.json
+run_command "vcf2genomicsdb $WORKSPACE/loader.json" OK
+sed -i -e 's|"row_based_partitioning": false|"vcf_header_filename": "'${WORKSPACE}'/vcfheader.vcf"|g' $WORKSPACE/loader.json
+run_command "vcf2genomicsdb $WORKSPACE/loader.json" OK
+sed -i -e 's|"workspace": "'${WORKSPACE}'"|"workspace": "'${WORKSPACE}'", "vcf_header_filename": "'${WORKSPACE}'/vcfheader.vcf", "vcf_output_filename": "'${WORKSPACE}'/out"|g' $WORKSPACE/loader.json
+run_command "vcf2genomicsdb $WORKSPACE/loader.json" OK
+
+# Test with too small size_per_column_partition
+sed -i -e 's/"size_per_column_partition": 700/"size_per_column_partition": 70/g' $WORKSPACE/loader.json
+run_command "vcf2genomicsdb $WORKSPACE/loader.json" OK
+sed -i -e 's/"size_per_column_partition": 70/"size_per_column_partition": 0/g' $WORKSPACE/loader.json
+run_command "vcf2genomicsdb $WORKSPACE/loader.json" ERR
+
 # Test --progress switch with an interval
+create_template_loader_json
 run_command "vcf2genomicsdb_init -w $WORKSPACE -S $SAMPLE_DIR -o -t $TEMPLATE"
 run_command "vcf2genomicsdb --progress=2 $WORKSPACE/loader.json"
 run_command "vcf2genomicsdb_init -w $WORKSPACE -S $SAMPLE_DIR -o -t $TEMPLATE"
@@ -317,6 +391,11 @@ create_template_loader_json -5 -5
 run_command "vcf2genomicsdb_init -w $WORKSPACE -s $SAMPLE_LIST -o -t $TEMPLATE"
 run_command "vcf2genomicsdb $WORKSPACE/loader.json" ERR
 
+if [[ $DISABLE_MPI == 1 ]]; then
+  echo "Build with DISABLE_MPI option set, not running gt_mpi_gather tests"
+  exit 0
+fi
+
 # Sanity test gt_mpi_gather
 create_sample_list t0.vcf.gz
 run_command_and_check_results "vcf2genomicsdb_init -w $WORKSPACE -s $SAMPLE_LIST -o" 1 85 24 85 "#30"
@@ -327,38 +406,110 @@ run_command "gt_mpi_gather -A "non-existent-array" $WORKSPACE/loader.json -j $QU
 run_command "gt_mpi_gather -l $WORKSPACE/loader.json -j $QUERY_JSON --print-AC"
 run_command "gt_mpi_gather -l $WORKSPACE/loader.json -j $QUERY_JSON --print-calls"
 
-# Fail if there is no reference genome file specified
-run_command "gt_mpi_gather -l $WORKSPACE/loader.json -j $QUERY_JSON --produce-Broad-GVCF" ERR
+# Test gt_mpi_gather with a partition not imported
+run_command "gt_mpi_gather --rank 1 -l $WORKSPACE/loader.json -j $QUERY_JSON" ERR
 
-# Fail if the reference genome is pointing to a non-existent file
-create_query_json "non-exisitent-reference-genome"
-run_command "gt_mpi_gather -l $WORKSPACE/loader.json -j $QUERY_JSON --produce-Broad-GVCF" ERR
-
-# Fail if the reference genome cannot be parsed by htslib for any reason
-create_query_json "$WORKSPACE/loader.json"
-run_command "gt_mpi_gather -l $WORKSPACE/loader.json -j $QUERY_JSON --produce-Broad-GVCF" ERR
-
-# Run with valid reference genome successfully
-create_query_json $REFERENCE_GENOME
-run_command "gt_mpi_gather -l $WORKSPACE/loader.json -j $QUERY_JSON --produce-Broad-GVCF"
-
-# Test consolidate_genomicsdb_array and gt_mpi_gather after consolidation
-diff_gt_output() {
+# Diff results from two gt_mpi_gather invocations
+# $1 Results from 1st invocation
+# $2 Results from 2nd invocation
+# $3 Expected Result(OK or ERR)
+# $4 Error Message
+diff_gt_mpi_gather_output() {
   sed "1 d" $1 > $1.cut
+  sed -i -e '/MD5/d' $1.cut
   sed "1 d" $2 > $2.cut
+  sed -i -e '/MD5/d' $2.cut
   DIFF_RESULT=$(diff $1.cut $2.cut)
-  if [[ ! -z  $DIFF_RESULT ]]; then
-     echo "gt_mpi_gather results for Test $3 differ after consolidation"
-     STATUS=1
+  if [[ ! -z  $DIFF_RESULT && $3 == OK ]]; then
+    echo "gt_mpi_gather results differ : $4"
+    echo "First output:"
+    cat $1.cut
+    echo "Second output:"
+    cat $2.cut
+    STATUS=1
+  elif [[ -z $DIFF_RESULT && $3 == ERR ]]; then
+    echo "gt_mpi_gather results should differ but were identical : $4"
+    STATUS=1
   fi
 }
 
+# $1 : intervals
+test_bypass_intersecting_intervals_phase() {
+  create_query_json_with_intervals "$1"
+  time_command "gt_mpi_gather -l $WORKSPACE/loader.json -j $QUERY_JSON --print-calls"
+  DIFF=$TIME
+  mv -f $TEMP_DIR/output $TEMP_DIR/output.without_bypass
+  create_query_json_with_intervals "$1" "\"bypass_intersecting_intervals_phase\": true"
+  time_command "gt_mpi_gather -l $WORKSPACE/loader.json -j $QUERY_JSON --print-calls"
+  if [ $DIFF -lt $TIME ]; then
+    echo "Bypass intersecting intervals phase took longer than the normal two pass iterator"
+    STATUS=1
+  fi
+}
+
+# Test bypass of intersecting intervals phase in the genomicsdb iterators with no output
+INTERVALS="\"query_contig_intervals\": [{\"contig\": \"1\", \"begin\": 1, \"end\":100 }]"
+test_bypass_intersecting_intervals_phase "$INTERVALS"
+diff_gt_mpi_gather_output $TEMP_DIR/output.without_bypass $TEMP_DIR/output OK "Setting bypass intersecting intervals should not affect gt_mpi_gather for $INTERVALS for t0.vcf.gz"
+
+# Test bypass of intersecting intervals phase in the genomicsdb iterators
+INTERVALS="\"query_contig_intervals\": [{\"contig\": \"1\", \"begin\": 1, \"end\":12142 }]"
+test_bypass_intersecting_intervals_phase "$INTERVALS"
+diff_gt_mpi_gather_output $TEMP_DIR/output.without_bypass $TEMP_DIR/output OK "Setting bypass intersecting intervals should not affect gt_mpi_gather for $INTERVALS for t0.vcf.gz"
+
+# Test bypass of intersecting intervals phase in the genomicsds iterators with two intervals
+INTERVALS="\"query_contig_intervals\": [{\"contig\": \"1\", \"begin\": 1, \"end\":12142 }, {\"contig\": \"1\", \"begin\": 17385 }]"
+test_bypass_intersecting_intervals_phase "$INTERVALS"
+diff_gt_mpi_gather_output $TEMP_DIR/output.without_bypass $TEMP_DIR/output OK "Setting bypass intersecting intervals should not affect gt_mpi_gather for $INTERVALS for t0.vcf.gz"
+
+# Test bypass of intersecting intervals phase with indels
+create_sample_list t6_asa.vcf.gz
+run_command "vcf2genomicsdb_init -w $WORKSPACE -s $SAMPLE_LIST -o"
+run_command "vcf2genomicsdb $WORKSPACE/loader.json"
+
+# Test bypass of intersecting intervals phase with indels with query interval encompassing entire indel
+INTERVALS="\"query_contig_intervals\": [{\"contig\": \"1\", \"begin\": 8029500, \"end\": 8029510 }]"
+test_bypass_intersecting_intervals_phase "$INTERVALS"
+diff_gt_mpi_gather_output $TEMP_DIR/output.without_bypass $TEMP_DIR/output OK "Setting bypass intersecting intervals should not affect gt_mpi_gather results to be different for $INTERVALS for t6_asa.vcf.gz as the entire indel is included in the query interval"
+
+# Test bypass of intersecting intervals phase with indels with query interval overlapping one position in the indel
+INTERVALS="\"query_contig_intervals\": [{\"contig\": \"1\", \"begin\": 8029501 }]"
+test_bypass_intersecting_intervals_phase "$INTERVALS"
+diff_gt_mpi_gather_output $TEMP_DIR/output.without_bypass $TEMP_DIR/output ERR "Setting bypass intersecting intervals should cause gt_mpi_gather to be different for $INTERVALS for t6_asa.vcf.gz as the start of the indel is not included in the query interval"
+
+# Test bypass of intersecting intervals phase with indels with query interval overlapping start of indel
+INTERVALS="\"query_contig_intervals\": [{\"contig\": \"1\", \"begin\": 8029500, \"end\": 8029501 }]"
+test_bypass_intersecting_intervals_phase "$INTERVALS"
+diff_gt_mpi_gather_output $TEMP_DIR/output.without_bypass $TEMP_DIR/output OK "Setting bypass intersecting intervals should not affect gt_mpi_gather results to be different for $INTERVALS for t6_asa.vcf.gz as the start of the indel is included in the query interval"
+
+# Test exception thrown with vcfs having overlapping variants, where the call is neither a deletion, mnv or a reference block
+create_sample_list t9_unhandled_overlapping_variants.vcf.gz
+run_command "vcf2genomicsdb_init -w $WORKSPACE -s $SAMPLE_LIST -o"
+run_command "vcf2genomicsdb -r 12 $WORKSPACE/loader.json"
+create_query_json_with_intervals "\"query_contig_intervals\": [{\"contig\": \"13\", \"begin\": 1, \"end\":115169878 }]" "\"reference_genome\" : \"$REFERENCE_GENOME\""
+sed -i -e 's/"RAW_MQ",//g' $QUERY_JSON
+run_command "gt_mpi_gather --rank 12 -l $WORKSPACE/loader.json -j $QUERY_JSON --produce-Broad-GVCF" ERR
+
+# Fail if the reference genome cannot be parsed by htslib for any reason
+create_sample_list t0.vcf.gz
+run_command "vcf2genomicsdb_init -w $WORKSPACE -s $SAMPLE_LIST -o"
+run_command "vcf2genomicsdb $WORKSPACE/loader.json"
+
+# Fail when the reference file is not a reference file
+create_query_json "\"reference_genome\" : $WORKSPACE/loader.json"
+run_command "gt_mpi_gather -l $WORKSPACE/loader.json -j $QUERY_JSON --produce-Broad-GVCF" ERR
+
+# Run with valid reference genome successfully
+create_query_json "\"reference_genome\" : \"$REFERENCE_GENOME\""
+run_command "gt_mpi_gather -l $WORKSPACE/loader.json -j $QUERY_JSON --produce-Broad-GVCF"
+
+# Test consolidate_genomicsdb_array and gt_mpi_gather after consolidation
 run_consolidation_and_check_results() {
   run_command "gt_mpi_gather -l $WORKSPACE/loader.json -j $QUERY_JSON --print-AC"
   mv -f $TEMP_DIR/output $TEMP_DIR/output.no.consolidation
   run_command "$1"
   run_command "gt_mpi_gather -l $WORKSPACE/loader.json -j $QUERY_JSON --print-AC"
-  diff_gt_output  $TEMP_DIR/output.no.consolidation $TEMP_DIR/output $2
+  diff_gt_mpi_gather_output  $TEMP_DIR/output.no.consolidation $TEMP_DIR/output $2 OK "After consolidation"
 }
 
 ARRAY="1\$1\$249250621"
